from __future__ import annotations

import tkinter
from functools import partial
from pathlib import Path
from tkinter import ttk
import ctypes as ct
import os
os.system("pip install darkdetect")
os.system("pip install platform")
import darkdetect
import platform

TCL_THEME_FILE_PATH = Path(__file__).with_name("sv.tcl").absolute()

def center(win):
    """
    centers a tkinter window
    :param win: the main window or Toplevel window to center
    """
    win.update_idletasks()
    width = win.winfo_width()
    frm_width = win.winfo_rootx() - win.winfo_x()
    win_width = width + 2 * frm_width
    height = win.winfo_height()
    titlebar_height = win.winfo_rooty() - win.winfo_y()
    win_height = height + titlebar_height + frm_width
    x = win.winfo_screenwidth() // 2 - win_width // 2
    y = win.winfo_screenheight() // 2 - win_height // 2
    win.geometry('{}x{}+{}+{}'.format(width, height, x, y))
    win.deiconify()
def _load_theme(style: ttk.Style) -> None:
    if not isinstance(style.master, tkinter.Tk):
        raise TypeError("root must be a `tkinter.Tk` instance!")

    if not hasattr(style.master, "_sv_ttk_loaded"):
        style.tk.call("source", str(TCL_THEME_FILE_PATH))
        style.master._sv_ttk_loaded = True  # type: ignore


def get_theme(root: tkinter.Tk | None = None) -> str:
    style = ttk.Style(master=root)
    _load_theme(style)

    theme = style.theme_use()
    return {"sun-valley-dark": "dark", "sun-valley-light": "light"}.get(theme, theme)


def set_theme(theme: str, root: tkinter.Tk | None = None) -> None:
    style = ttk.Style(master=root)
    _load_theme(style)

    theme = theme.lower()

    if theme not in {"dark", "light"}:
        raise RuntimeError(f"not a valid sv_ttk theme: {theme}")

    style.theme_use(f"sun-valley-{theme}")


def toggle_theme(root: tkinter.Tk | None = None) -> None:
    style = ttk.Style(master=root)
    _load_theme(style)

    set_theme("light" if style.theme_use() == "sun-valley-dark" else "dark")
def titlebar(root: tkinter.tk,theme):
    def dark_title_bar(window):
        window.update()
        DWMWA_USE_IMMERSIVE_DARK_MODE = 20
        set_window_attribute = ct.windll.dwmapi.DwmSetWindowAttribute
        get_parent = ct.windll.user32.GetParent
        hwnd = get_parent(window.winfo_id())
        rendering_policy = DWMWA_USE_IMMERSIVE_DARK_MODE
        value = 2
        value = ct.c_int(value)
        set_window_attribute(hwnd, rendering_policy, ct.byref(value), ct.sizeof(value))
    def light_title_bar(window):
        window.update()
        DWMWA_USE_IMMERSIVE_DARK_MODE = 20
        set_window_attribute = ct.windll.dwmapi.DwmSetWindowAttribute
        get_parent = ct.windll.user32.GetParent
        hwnd = get_parent(window.winfo_id())
        rendering_policy = DWMWA_USE_IMMERSIVE_DARK_MODE
        value = 0
        value = ct.c_int(value)
        set_window_attribute(hwnd, rendering_policy, ct.byref(value), ct.sizeof(value))
    if theme not in {"dark", "light"}:
            raise RuntimeError(f"not a valid sv_ttk theme: {theme}")
    else:
        if theme == "light":
            light_title_bar(root)
        elif theme == "dark":
            dark_title_bar(root)
def native_theme(root:tkinter.tk):
    set_theme("light",root)
    is_dark = darkdetect.isDark()
    if is_dark:
        titlebar(root,"dark")
        toggle_theme()
    else:
        titlebar(root,"light")
    center(root)

def system_check():
    if platform.system() == "Windows":
        print("All functionality available")
    else:
        print("Some features are ristricted.\nTheese features include:messagebox, native theme, and titlebar.\nPlease do not use theese functions.")
use_dark_theme = partial(set_theme, "dark")
use_light_theme = partial(set_theme, "light")

    



import os
os.system("pip install Pillow")
from winsound import MessageBeep, MB_ICONHAND, MB_ICONASTERISK
from ctypes import windll, c_int, byref, sizeof, c_char_p
from tkinter import Toplevel,Tk, X, Y, LEFT, RIGHT, BOTTOM, Frame
from tkinter.ttk import Button, Label
from PIL import Image, ImageTk
from darkdetect import isDark
from sv_ttk import set_theme
from os import getenv

systemroot = getenv("SYSTEMROOT") + "\\System32\\"
class NormalModel(Toplevel):
	""" This is a class for Normal Messagebox Model """
	def __init__(self, mtitle, mtext, mcommand, mtype, mclick = "OK"):
		""" Init this model"""
		super().__init__()
		
		self.master = Tk()
		self.master.withdraw()
		self.mtitle = mtitle # Title
		self.mtext = mtext # Infomation
		self.mtype = mtype.lower()
		if self.mtype == "showwarning": # Type icon
			MessageBeep(eval("MB_ICONHAND"))
			self.icon = systemroot + "SecurityAndMaintenance_Alert.png"
		elif self.mtype == "showerror":
			MessageBeep(eval("MB_ICONHAND"))
			self.icon = systemroot + "SecurityAndMaintenance_Error.png"
		else:
			MessageBeep(eval("MB_ICONASTERISK"))
			self.icon = systemroot + "SecurityAndMaintenance.png"
		load = Image.open(self.icon)   # Open image
		load = load.resize((55, 55)) # Resize the image
		image = ImageTk.PhotoImage(load)  # Read opened image
		
		self.title(self.mtitle) # Icon will set by master window
		self.iconphoto(False, image)
		self.minsize(375, 175)
		self.resizable(False, False)
		self.transient(self.master)
		self.attributes("-topmost", True)

		self.show = Frame(self)
		if not mcommand: self.command = self.mcommand
		else: self.command = mcommand
		self.cilck = Button(self.show, text = mclick, command = self.command, style = "Accent.TButton", width = 7)
		
		self.show.pack(side = BOTTOM, fill = X)
		self.cilck.pack(side = RIGHT, fill = X)
		self.image = Label(self, image = image)
		self.image.pack(side = LEFT, fill = X, padx = 25, pady = 10)
		self.info = Label(self, text = self.mtext)
		self.info.pack(side = LEFT, fill = X, padx = 25, pady = 10)
		
		try:
			if isDark(): 
				self.darkmode()
				self.show["bg"] = "#1c1c1a"
			else: 
				set_theme("dark")
				self.show["bg"] = "#f5f5f5"
		except: pass # The master window had theme
		self.mainloop()
		
		del load, image
			
	def mcommand(self):
		""" Normal Button Command"""
		self.destroy()
		return True
	
	def darkmode(self):
		""" Set window into darkmode"""
		set_theme("dark")
		self.update()
		DWMWA_USE_IMMERSIVE_DARK_MODE = 20
		set_window_attribute = windll.dwmapi.DwmSetWindowAttribute
		get_parent = windll.user32.GetParent
		hwnd = get_parent(self.winfo_id())
		rendering_policy = DWMWA_USE_IMMERSIVE_DARK_MODE
		value = 2
		value = c_int(value)
		set_window_attribute(hwnd, rendering_policy, byref(value), sizeof(value))
		self.withdraw()
		self.deiconify()

def showinfo(mtitle, mtext, mcommand = None):
	""" Showinfo Messagebox """
	handle = NormalModel(mtitle, mtext, mcommand, mtype = "showinfo")
	return True

def showwarning(mtitle, mtext, mcommand = None):
	""" Showwarning Messagebox """
	handle = NormalModel(mtitle, mtext, mcommand, mtype = "showwarning")
	return True
	
def showerror(mtitle, mtext, mcommand = None):
	""" Showerror Messagebox """
	handle = NormalModel(mtitle, mtext, mcommand, mtype = "showerror")
	return True

